package com.demo.phoneauthplugin

import android.app.Activity
import android.util.Log
import android.widget.Toast
import com.google.firebase.FirebaseException
import com.google.firebase.FirebaseTooManyRequestsException
import com.google.firebase.auth.FirebaseAuth
import com.google.firebase.auth.FirebaseAuthInvalidCredentialsException
import com.google.firebase.auth.PhoneAuthCredential
import com.google.firebase.auth.PhoneAuthProvider
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar
import java.util.concurrent.TimeUnit


class PhoneAuthPlugin(private var mContext: Activity,private var mMethodChannel: MethodChannel) : MethodCallHandler {


    val mFirebasAuth=FirebaseAuth.getInstance()
    val  TAG=PhoneAuthPlugin::class.java.simpleName
    var codeSent:String?=null



    companion object {
        @JvmStatic
        fun registerWith(registrar: Registrar) {
            val channel = MethodChannel(registrar.messenger(), "phone_auth_plugin")
            channel.setMethodCallHandler(PhoneAuthPlugin(registrar.activity(),channel))
        }
    }

    override fun onMethodCall(call: MethodCall, result: Result) {
        if (call.method == "getPlatformVersion") {
            result.success("Android ${android.os.Build.VERSION.RELEASE}")
        } else if (call.method == "SEND_CODE") {

            Log.e("IN_SEND_CODE", "in send code")
            val phone=call.argument<String>("PHONE_NUMBER")
            phone?.let {

                sendVericicationCOde(it)
            }

        } else if (call.method == "VERIFY_CODE") {

            val code=call.argument<String>("CODE")
            code?.let {

                verifyCode(it)
            }

        } else {
            result.notImplemented()
        }
    }

    val callbacks = object : PhoneAuthProvider.OnVerificationStateChangedCallbacks() {

        override fun onVerificationCompleted(credential: PhoneAuthCredential) {
            // This callback will be invoked in two situations:
            // 1 - Instant verification. In some cases the phone number can be instantly
            //     verified without needing to send or enter a verification code.
            // 2 - Auto-retrieval. On some devices Google Play services can automatically
            //     detect the incoming verification SMS and perform verification without
            //     user action.
            Log.d(TAG, "onVerificationCompleted:$credential")
//
        }

        override fun onVerificationFailed(e: FirebaseException) {
            // This callback is invoked in an invalid request for verification is made,
            // for instance if the the phone number format is not valid.
            Log.w(TAG, "onVerificationFailed", e)

            if (e is FirebaseAuthInvalidCredentialsException) {
                // Invalid request
                // ...
            } else if (e is FirebaseTooManyRequestsException) {
                // The SMS quota for the project has been exceeded
                // ...
            }

            // Show a message and update the UI
            // ...
        }

        override fun onCodeSent(
                verificationId: String?,
                token: PhoneAuthProvider.ForceResendingToken
        ) {
            // The SMS verification code has been sent to the provided phone number, we
            // now need to ask the user to enter the code and then construct a credential
            // by combining the code with a verification ID.
            Log.d(TAG, "onCodeSent:" + verificationId!!)

            // Save verification ID and resending token so we can use them later
            codeSent = verificationId
//            resendToken = token

            // ...
        }
    }

    private fun  sendVericicationCOde(phoneNumber:String){

        PhoneAuthProvider.getInstance().verifyPhoneNumber(
                phoneNumber,      // Phone number to verify
                60,               // Timeout duration
                TimeUnit.SECONDS, // Unit of timeout
                mContext,             // Activity (for callback binding)
                callbacks) // OnVerificationStateChangedCallbacks

    }

    private fun verifyCode(code:String){
//        val code=edtVerifyOtp.text.toString()
        val credential = PhoneAuthProvider.getCredential(codeSent!!, code)
        signInWithPhoneAuthCredential(credential)

    }

    private fun signInWithPhoneAuthCredential(credential: PhoneAuthCredential) {
        mFirebasAuth.signInWithCredential(credential)
                .addOnCompleteListener(mContext) { task ->
                    if (task.isSuccessful) {
                        // Sign in success, update UI with the signed-in user's information
                        Log.d(TAG, "signInWithCredential:success")

                        val user = task.result?.user

                       Toast.makeText(mContext,"Code verifed",Toast.LENGTH_SHORT).show()
                        // ...
                    } else {
                        // Sign in failed, display a message and update the UI
                        Log.w(TAG, "signInWithCredential:failure", task.exception)
                        if (task.exception is FirebaseAuthInvalidCredentialsException) {
                            // The verification code entered was invalid
                        }

                        Toast.makeText(mContext,"Code not verifed",Toast.LENGTH_SHORT).show()

                    }
                }
    }
}
